/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Bancisor Mihai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 759
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "Communication.h"
#include "AD7156.h"
#include "Console.h"
#include "Delays.h"
/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
_FOSCSEL(FNOSC_PRIPLL);
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF  & POSCMD_XT);    /*!< Clock Switching Enabled and Fail Safe Clock Monitor is disabled*/
                                                    /*!< OSC2 Pin Function: OSC2 is Clock Output*/
                                                    /*!  Primary Oscillator Mode: XT Crystal*/
_FWDT(FWDTEN_OFF); 			            /*!< Watchdog Timer Enabled/disabled by user software*/

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/* List of available commands */
const char* commandsList[] = {"help?",
                              "range=",
                              "power=",
                              "channel1=",
                              "channel2=",
                              "samples?",
                              "buttons?",
                              "reset="};
const char* commandsDescription[] = {
    "  -  Displays all available commands.",
    "  -  Selects the range of the channels.\r\n Accepted values: \r\n 0: 0 - 2 pF \r\n 1: 0 - 0.5 pF \r\n 2: 0 - 1 pF \r\n 3: 0 - 4 pF",
    "  -  Selects the power mode of the device.\r\n Accepted values: \r\n 0: Idle \r\n 1: Continuous conversions \r\n 2: Single conversion \r\n 3: Power-down",
    "  -  Enables or disabled conversions on channel 1. Accepted values 0-1.",
    "  -  Enables or disabled conversions on channel 2. Accepted values 0-1.",
    "  -  Reads 50 samples form the enabled channels and displays the data as pF.",
    "  -  Checks if any of the two buttons are pressed or released.",
    "  -  Resets the device. Accepted value: 1."};
/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
double        commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converterd value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    short  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    short  i = sizeof(localBuf) - 1;

    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > sizeof(localBuf) - 4)
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
	*buf = 0;
}
/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
int main(void)
{
    /*! Configure PLL prescaler, PLL postscaler, PLL divisor. */
    /*! Fosc = 80MHz, Fcy = 40MHz, 40MIPS. */
    PLLFBD = 38;            // M = PLLFBD + 2
    CLKDIVbits.PLLPOST = 0; // N2 = 2
    CLKDIVbits.PLLPRE = 0;  // N1 = 2

    /* Variables holding information about the device */
    unsigned char  status        = 0; // The content of the STATUS register.
    unsigned char  btnEvent      = 0; // Holds the number of button actions.
    unsigned char  prevBtn1State = 0; // Previous state of Button1(ON/OFF).
    unsigned char  prevBtn2State = 0; // Previous state of Button2(ON/OFF).
    unsigned char  ch1Enable     = 1; // Enable state of channel 1.
    unsigned char  ch2Enable     = 1; // Enable state of channel 2.
    float ch1 = 0;                    // Value of the capacitance measured(pF).
    float ch2 = 0;                    // Value of the capacitance measured(pF).
    /* Temporary variables */
    char  tempString[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    long           intCommand     = 0;
    long           samplesNr      = 0;

    /*! Wait for PLL to lock. */
    while(OSCCONbits.COSC != 0b011);
    while(OSCCONbits.LOCK != 1);

    T1CONbits.TON = 0;      // Disable Timer.
    T1CONbits.TCS = 0;      // Select internal instruction cycle clock.
    T1CONbits.TGATE = 0;    // Disable Gated Timer mode.
    T1CONbits.TCKPS = 0b01; // Select 1:1 Prescaler.
                            // 1 tick ~ 0.2 us.
    TMR1 = 0x00;            // Clear timer register.
    PR1 = 0xFFFF;           // Load the period value.
    T1CONbits.TON = 1;      // Start Timer.

    /* Initialize the UART communication peripheral. */
    UART_Init(9600);
    /* Initialize the AD7156 device. */
    if(AD7156_Init())
    {
        CONSOLE_WriteString("AD7156 OK");
        UART_Write(0x0D);
    }
    else
    {
        CONSOLE_WriteString("AD7156 Error");
        UART_Write(0x0D);
    }
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    // "help?" command
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))   // "range=" command
            {
                intCommand = (long)commandParam;
                /* Check if the parameter is valid. */
                if(intCommand < 0)
                {
                    intCommand = 0;
                }
                else if(intCommand > 3)
                {
                    intCommand = 3;
                }
                AD7156_SetRange(AD7156_CHANNEL1, intCommand);
                AD7156_SetRange(AD7156_CHANNEL2, intCommand);
                /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
             if((command == 2) && (commandType != 0))   // "power=" command
             {
                intCommand = (long)commandParam;
                /* Check if the parameter is valid. */
                if(intCommand < 0)
                {
                    intCommand = 0;
                }
                else if(intCommand > 3)
                {
                    intCommand = 3;
                }
                AD7156_SetPowerMode(intCommand);
                /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
             }
             if((command == 3) && (commandType != 0))  // "channe1=" command
             {
                 intCommand = (long)commandParam;
                 /* Check if the parameter is valid. */
                 if((commandParam == 1) || (commandParam == 0))
                 {
                    AD7156_ChannelState(AD7156_CHANNEL1, intCommand);
                    ch1Enable = intCommand;
                 }
                 else
                 {
                     intCommand = 0;
                     ch1Enable = 0;
                 }
                 /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
             }
             if((command == 4) && (commandType != 0))  // "channel2=" command
             {
                intCommand = (long)commandParam;
                 /* Check if the parameter is valid. */
                 if((commandParam == 1) || (commandParam == 0))
                 {
                    AD7156_ChannelState(AD7156_CHANNEL2, intCommand);
                    ch2Enable = intCommand;
                 }
                 else
                 {
                     intCommand = 0;
                     ch2Enable = 0;
                 }
                 /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, intCommand, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
             }
            if((command == 5) && (commandType != 0))  // "samples?" command
            {
                /* Display 50 samples from channel1 and channel2. */
                for(samplesNr = 0; samplesNr < 50; samplesNr++)
                {
                    if((ch1Enable == 1) && (ch2Enable == 0))
                    {
                        ch1 = AD7156_ReadChannelCap(AD7156_CHANNEL1);
                        ch2 = 0;
                    }else if((ch1Enable == 0) && (ch2Enable == 1))
                    {
                        ch1 = 0;
                        ch2 = AD7156_ReadChannelCap(AD7156_CHANNEL2);
                    }
                    else if((ch1Enable == 1) && (ch2Enable == 1))
                    {
                        AD7156_ReadCapacitance(&ch1, &ch2);
                    }
                    FloatToString(tempString, ch1);
                    CONSOLE_WriteString("Ch1: ");
                    CONSOLE_WriteString(tempString);
                    CONSOLE_WriteString(" pF   ");
                    CONSOLE_WriteString("Ch2: ");
                    FloatToString(tempString, ch2);
                    CONSOLE_WriteString(tempString);
                    CONSOLE_WriteString(" pF");
                    UART_Write(0xD);
                }
            }
            if((command == 6) && (commandType != 0))  // "buttons?" command
            {
                /* Configure device. */
                AD7156_SetPowerMode(AD7156_CONV_MODE_IDLE);
                AD7156_SetThresholdMode(AD7156_THR_MODE_NEGATIVE,
                                        AD7156_FIXED_THRESHOLD);
                AD7156_SetThreshold(AD7156_CHANNEL1, 1.0); // Threshold = 1.0pF
                AD7156_SetThreshold(AD7156_CHANNEL2, 1.0); // Threshold = 1.0pF
                AD7156_SetPowerMode(AD7156_CONV_MODE_CONT_CONV);
                /* Start polling buttons(channels) state. */
                CONSOLE_WriteString("Start button polling.");
                btnEvent = 0;
                prevBtn1State = 0;
                prevBtn2State = 0;
                /* After 15 button actions the execution returns to the menu. */
                while(btnEvent <= 15)
                {
                    /* Read the STATUS register. */
                    AD7156_GetRegisterValue(&status, AD7156_REG_STATUS, 1);
                    /* Check for changes in the button state. */
                    if(((status & AD7156_STATUS_OUT1) !=0) &&
                       (prevBtn1State == 0))
                    {
                        CONSOLE_WriteString("BTN1 - pressed");
                        UART_Write(0xD);
                        prevBtn1State = 1;
                        btnEvent++;
                    }
                    else if(((status & AD7156_STATUS_OUT1) == 0) &&
                            (prevBtn1State == 1))
                    {
                        CONSOLE_WriteString("BTN1 - released");
                        UART_Write(0xD);
                        prevBtn1State = 0;
                        btnEvent++;
                    }
                    if(((status & AD7156_STATUS_OUT2) !=0) &&
                       (prevBtn2State == 0))
                    {
                        CONSOLE_WriteString("BTN2 - pressed");
                        UART_Write(0xD);
                        prevBtn2State = 1;
                        btnEvent++;
                    }
                    else if(((status & AD7156_STATUS_OUT2) == 0) &&
                            (prevBtn2State == 1))
                    {
                        CONSOLE_WriteString("BTN2 - released");
                        UART_Write(0xD);
                        prevBtn2State = 0;
                        btnEvent++;
                    }
                }
                CONSOLE_WriteString("Button polling finished.");
            }
            if((command == 7) && (commandType != 0))  // "reset=" command
            {
                if(commandParam == 1)
                {
                    AD7156_Reset();
                }
                else
                {
                    commandParam = 0;
                }
                /* Send feedback to user. */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, (long)commandParam, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
        }
        if(invalidCommand == commandsNumber)
        {
            /* Send feedback to user */
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    }
}